@@ -26,15 +26,16 @@ use vars qw($VERSION @EXPORT @EXPORT_OK %EXPORT_TAGS @ISA);
 #use LWP::Protocol::https;
 #use Data::Dumper;
 
-$VERSION = '0.14';
+$VERSION = '0.15';
 @ISA = qw(Exporter);
 @EXPORT = qw();
 @EXPORT_OK = qw(
 sethtml clearhtml add_details add_google add_google_search add_orcid add_fromfile add_dblp add_pubmed
-send_resp print print_nodoi num num_nodoi getref getref_nodoi append
+send_resp print print_nodoi num num_nodoi getref getref_nodoi append add_bibtex
 );
 %EXPORT_TAGS = (all => \@EXPORT_OK);
 
+####################################################################################
 sub new {
     my $self;
     # defaults
@@ -51,16 +52,19 @@ sub new {
     return $self;
 }
 
+####################################################################################
 sub sethtml {
   my $self = shift @_;
   $self->{html}=1;
 }
 
+####################################################################################
 sub clearhtml {
   my $self = shift @_;
   $self->{html}=0;
 }
 
+####################################################################################
 sub _err {
   my ($self, $str) = @_;
   if ($self->{html}) {
@@ -70,6 +74,7 @@ sub _err {
   }
 }
 
+####################################################################################
 sub _split_duplicates {
   # split list of references into three lists: one with refs that have unique doi's, one with no doi's
   #and one with all the rest (with duplicate doi's)
@@ -95,6 +100,7 @@ sub _split_duplicates {
   $self->{refs} = \@newrefs;
 }
 
+####################################################################################
 sub append {
   # add new reference to end of existing list
   my $self = shift @_;
@@ -102,6 +108,7 @@ sub append {
   push @{$self->{refs}}, $ref;
 }
 
+####################################################################################
 sub add_details {
   # given an array of raw strings, try to convert into paper references
   
@@ -115,6 +122,7 @@ sub add_details {
   $self->_split_duplicates();
 }
 
+####################################################################################
 sub add_google {
   # scrape paper details from google scholar personal page -- nb: no doi info on google, so use crossref.org to obtain this
   # nb: doesn't work with google scholar search results
@@ -153,6 +161,7 @@ sub add_google {
   }
 }
 
+####################################################################################
 sub add_google_search {
   # scrape paper details from google scholar search results -- *not* from persons scholar home page
 
@@ -184,6 +193,24 @@ sub add_google_search {
   }
 }
 
+####################################################################################
+sub _dblp_setauth {
+  my $self = shift @_; my $r = shift @_; my $cite = shift @_;
+  
+  if (ref($cite->{'author'}) eq "HASH") {
+    $r->_setauthcount(1);
+    $r->_setauth(1,$cite->{'author'});
+  } else {
+    my $count = 0;
+    foreach my $au (@{$cite->{'author'}}) {
+      $count++;
+      $r->_setauth($count, $au);
+    }
+    $r->_setauthcount($count);
+  }
+}
+
+####################################################################################
 sub add_dblp {
   # get details using DBLP XML API
   
@@ -210,37 +237,42 @@ sub add_dblp {
     my $num=0;
     foreach my $c (@{$cites[0]}) {
       $num++; if ($maxnum>0 && $num>$maxnum) {last;}  # mainly for testing
-      my $cite;
       my @k = keys %{$c};
-      $cite = $c->{$k[0]};
+      my $cite = $c->{$k[0]};
       my $ee = $cite->{'ee'};
       if ($ee =~ m/dx.doi.org/) {
         # we have a DOI, lets call crossref
         $ee =~ s/http:\/\/dx.doi.org\///;
-        $self->add_details($ee);
-        next; # move on to next record
+        my $r = Bib::CrossRef->new;
+        $r->parse_text($ee);
+        if ($r->score >=1) {
+          if (!defined $r->authcount || $r->authcount==0) {
+            # shouldn't happen, but sometimes doi data lacks authors so use dblp data
+            $self->_dblp_setauth($r,$cite);
+          }
+          $self->append($r);
+          next; # move on to next record
+        }
       }
-      my $temp = $cite->{'year'};
-      $temp .= ' '.$cite->{'title'};
+      my $jtitle='';
       if (defined $cite->{'journal'}) {
-        $temp .= ' '.$cite->{'journal'};
+        $jtitle = $cite->{'journal'};
       } elsif (defined $cite->{'booktitle'}) {
-        $temp .= ' '.$cite->{'booktitle'};
+        $jtitle = $cite->{'booktitle'};
       }
-      my $auth='';
+      my $temp = $cite->{'year'}.' '.$cite->{'title'}.' '.$jtitle. ' ';
       if (ref($cite->{'author'}) eq "HASH") {
-        $auth = $cite->{'author'};
+        $temp .= $cite->{'author'};
       } else {
-        foreach my $au (@{$cite->{'author'}}) { $auth .= $au.", ";}
+        foreach my $au (@{$cite->{'author'}}) { $temp .= $au.", ";}
       }
       my $r = Bib::CrossRef->new;
-      $r->parse_text($auth.' '.$temp);
+      $r->parse_text($temp);
       if ($r->score >= 1) {
         # found an ok match, lets use it
         $self->append($r);
         next; # move on
       }
-      #next; # stop here for now
       
       # we got a poor match, lets use the rest of the dblp data
       $r = Bib::CrossRef->new;
@@ -255,27 +287,8 @@ sub add_dblp {
       } else {
         $r->_setgenre($k[0]);
       }
-      $r->_setdate($cite->{'year'});
-      $r->_setatitle($cite->{'title'});
-      if (defined $cite->{'journal'}) {
-        $r->_setjtitle($cite->{'journal'});
-      } elsif (defined $cite->{'booktitle'}) {
-        $r->_setjtitle($cite->{'booktitle'});
-      }
-      $auth='';
-      if (ref($cite->{'author'}) eq "HASH") {
-        $r->_setauthcount(1);
-        $r->_setauth(1,$cite->{'author'});
-        $auth = $cite->{'author'};
-      } else {
-        my $count = 0;
-        foreach my $au (@{$cite->{'author'}}) {
-          $count++;
-          $r->_setauth($count, $au);
-          $auth .= $au.", ";
-        }
-        $r->_setauthcount($count);
-      }
+      $r->_setdate($cite->{'year'}); $r->_setatitle($cite->{'title'}); $r->_setjtitle($jtitle);
+      $self->_dblp_setauth($r,$cite);
       if (defined $cite->{'volume'}) {$r->_setvolume($cite->{'volume'});}
       if (defined $cite->{'number'}) {$r->_setissue($cite->{'number'});}
       if (defined $cite->{'pages'}) {
@@ -283,11 +296,9 @@ sub add_dblp {
         if (defined $bits[0]) {$r->_setspage($bits[0]);}
         if (defined $bits[1]) {$r->_setepage($bits[1]);}
       }
-      if (($cite->{'ee'} =~ m/^http:\/\//))  {
-        $r->_seturl($cite->{'ee'});
-      }
+      if (($cite->{'ee'} =~ m/^http:\/\//))  {$r->_seturl($cite->{'ee'});}
       $r->_setscore(1);
-      $r->_setquery($auth." ".$temp);
+      $r->_setquery($temp);
       # add manually constructed record
       $self->append($r);
     }
@@ -297,60 +308,91 @@ sub add_dblp {
   }
 }
 
-sub _get_orcid_doi {
+####################################################################################
+sub _orcid_getdoi {
+  # extract DOI from an orcid entry
   my $cite = shift @_;
   
   my $doi='';
-  if (ref($cite->{'work-external-identifiers'}->{'work-external-identifier'}) eq "HASH") {
+  my $c = $cite->{'work-external-identifiers'}->{'work-external-identifier'};
+  if (ref($c) eq "HASH") {
     # a single value
-    my $id = $cite->{'work-external-identifiers'}->{'work-external-identifier'};
-    if ($id->{'work-external-identifier-type'} =~ m/doi/) {
+    if ($c->{'work-external-identifier-type'} =~ m/doi/) {
       # and its a DOI
-      $doi =  $id->{'work-external-identifier-id'};
-    } else {
-      #print("Note: no DOI in ORCID for:".$cite->{'work-title'}->{'title'}.", only ".$id->{'work-external-identifier-type'}."=".$id->{'work-external-identifier-id'}."\n");
+      $doi =  $c->{'work-external-identifier-id'};
     }
   } else {
     # multiple values
-    my $found = 0; my $types;
-    foreach my $id (@{$cite->{'work-external-identifiers'}->{'work-external-identifier'}}) {
+    foreach my $id (@{$c}) {
       if ($id->{'work-external-identifier-type'} =~ m/doi/) {
         # its a DOI
         $doi =  $id->{'work-external-identifier-id'};
-        $found=1;
         last; # exit loop
       }
-      $types = $types.$id->{'work-external-identifier-type'}.'='.$id->{'work-external-identifier-id'}." ";
-    }
-    if (!$found) {
-      #print("Note: no DOI in ORCID for:".$cite->{'work-title'}->{'title'}.", only $types \n");
     }
   }
   return $doi;
 }
 
-sub _get_orcid_auth {
+####################################################################################
+sub _orcid_getauth {
+  # generate an author string from an orcid entry
   my $cite = shift @_;
 
   my $auth='';
-  if (ref($cite->{'work-contributors'}->{'contributor'}) eq "HASH") {
+  my $c = $cite->{'work-contributors'}->{'contributor'};
+  if (ref($c) eq "HASH") {
     # single author
-    my $au = $cite->{'work-contributors'}->{'contributor'};
-    if ($au->{'contributor-attributes'}->{'contributor-role'} =~ m/author/) {$auth = $au->{'credit-name'}->{'content'};}
+    if ($c->{'contributor-attributes'}->{'contributor-role'} =~ m/author/) {$auth = $c->{'credit-name'}->{'content'};}
   } else {
     # multiple authors
-    foreach my $au (@{$cite->{'work-contributors'}->{'contributor'}}) {
+    foreach my $au (@{$c}) {
       if ($au->{'contributor-attributes'}->{'contributor-role'} =~ m/author/) {$auth .= $au->{'credit-name'}->{'content'}.", ";}
     }
   }
   return $auth;
 }
 
+####################################################################################
+sub _orcid_setauth {
+  # use an orcid entry to set citation author list (using orcid bibtex data if appropriate)
+  my $self = shift @_; my $r = shift @_;
+  my $cite = shift @_; my $entry = shift @_;
+  
+  my $c = $cite->{'work-contributors'}->{'contributor'};
+  if (defined $c) {
+    # we have an orcid author entry
+    my $authcount=0;
+    if (ref($c) eq "HASH") {
+      # single author
+      if ($c->{'contributor-attributes'}->{'contributor-role'} =~ m/author/) {
+        $authcount++;
+        $r->_setauth($authcount,$c->{'credit-name'}->{'content'});
+      }
+    } else {
+      # multiple authors
+      foreach my $au (@{$c}) {
+        if ($au->{'contributor-attributes'}->{'contributor-role'} =~ m/author/) {
+          $authcount++;
+          $r->_setauth($authcount,$au->{'credit-name'}->{'content'});
+        }
+      }
+    }
+    $r->_setauthcount($authcount);
+    if ($authcount>0) {return;} # found some authors, finish up
+  }
+  # no author info, lets see if bibtex has any author info
+  if (defined $entry) {
+    $self->_bibtex_setauth($r,$entry);
+  }
+}
+
+####################################################################################
 sub add_orcid {
   # get paper details from orcid using API
   
-  my $self = shift @_;
-  my $orcid_id = shift @_;
+  my $self = shift @_; my $orcid_id = shift @_;
+  
   my $ua = LWP::UserAgent->new;
   my $req = HTTP::Request->new(GET => "http://pub.orcid.org/$orcid_id/orcid-works/");
   my $res = $ua->request($req);
@@ -360,92 +402,53 @@ sub add_orcid {
     my $data = $xs->XMLin($res->decoded_content);
     my @cites = $data->{'orcid-profile'}->{'orcid-activities'}->{'orcid-works'}->{'orcid-work'};
     foreach my $cite (@{$cites[0]}) {
-      my $ref={};
-      my $doi = _get_orcid_doi($cite);
-      if (defined $doi) {
-        # use DOI to search.crossref.org
-        my $r = Bib::CrossRef->new;
-        $r->parse_text($doi);
-        $self->append($r);
-        next;  # move on
-      }
-      # use title etc to search.crossref.org
-      my $temp='';
-      if (exists $cite->{'publication-date'}->{'year'}) {$temp = $cite->{'publication-date'}->{'year'};}
-      if (exists $cite->{'work-title'}->{'title'}) {$temp .= ' '.$cite->{'work-title'}->{'title'};}
-      if (exists $cite->{'journal-title'}) {$temp = ' '.$cite->{'journal-title'};}
-      my $auth = _get_orcid_auth($cite);
-      my $r = Bib::CrossRef->new;
-      $r->parse_text($auth.' '.$temp);
-      if ($r->score >= 1) {
-        # found an ok match, lets use it
-        $self->append($r);
-        next; # move on
-      }
-    
-      #next; # stop here for now
-
-      # for a poor match, try to extract rest of info from orcid
-      $r = Bib::CrossRef->new;
-      
-      if (exists $cite->{'work-title'}->{'title'}) {$r->_setatitle($cite->{'work-title'}->{'title'});}
-      if (exists $cite->{'journal-title'}) {$r->_setstitle($cite->{'journal-title'});}
-      if (exists $cite->{'publication-date'}->{'year'}) {$r->_setdate($cite->{'publication-date'}->{'year'});}
-      if (exists $cite->{'work-type'}) {$r->_setgenre($cite->{'work-type'});}
-      
-      my $authcount=0; $auth='';
-      if (ref($cite->{'work-contributors'}->{'contributor'}) eq "HASH") {
-        # single author
-        my $au = $cite->{'work-contributors'}->{'contributor'};
-        if ($au->{'contributor-attributes'}->{'contributor-role'} =~ m/author/) {
-          $authcount++;
-          $r->_setauth($authcount,$au->{'credit-name'}->{'content'});
-          $auth = $au->{'credit-name'}->{'content'};
-        }
-      } else {
-        # multiple authors
-        foreach my $au (@{$cite->{'work-contributors'}->{'contributor'}}) {
-          if ($au->{'contributor-attributes'}->{'contributor-role'} =~ m/author/) {
-            $authcount++;
-            $r->_setauth($authcount,$au->{'credit-name'}->{'content'});
-            $auth .= $au->{'credit-name'}->{'content'}.", ";
-          }
-        }
-      }
-      $r->_setauthcount($authcount);
-      
+      my $entry = undef;
       if ($cite->{'work-citation'}->{'work-citation-type'} =~ m/bibtex/) {
         # we have a bibtex reference, extract some extra info
-        # -- seems better to use crossref as this content can be messy
         my $bibtex = $cite->{'work-citation'}->{'citation'};
         open my $fh, '<', \$bibtex;
         my $parser = BibTeX::Parser->new($fh);
-        my $entry = $parser->next;
-        if ($entry->parse_ok) {
-          if (defined $entry->field('volume')) {$r->_setvolume($entry->field('volume'))};
-          if (defined $entry->field('issue')) {$r->_setissue($entry->field('issue'))};
-          if (defined $entry->field('pages')) {
-            my $pages = $entry->field('pages');
-            (my $s, my $e) = ($pages =~ /([0-9]+)-+([0-9]+)/ );
-            if (defined $s) { $r->_setspage($s); }
-            if (defined $e) { $r->_setepage($e); }
-          }
-          if (defined $entry->field('journal')) {
-            $r->_setjtitle($entry->field('journal'));
-          } elsif (defined $entry->field('booktitle')) {
-            $r->_setjtitle($entry->field('booktitle'));
-          }
-          my $jtitle = $r->jtitle;
-          if (defined $jtitle ) {
-            # tidy up
-            $jtitle =~ s/[\{\}]//g;
-            $jtitle =~ s/\\textquotesingle/\\'/g;
-            $r->_setjtitle($jtitle);
+        $entry = $parser->next;
+        if (!$entry->parse_ok) {$entry = undef;}
+      }
+      my $doi = _orcid_getdoi($cite);
+      if ((defined $doi) && (length($doi)>5)) { # we seem to have a DOI
+        # use DOI to search.crossref.org
+        my $r = Bib::CrossRef->new;
+        $r->parse_text($doi);
+        if ($r->score>=1) {
+          if (!defined $r->authcount || $r->authcount==0) {
+            # shouldn't happen, but sometimes doi data lacks authors so use orcid data
+            $self->_orcid_setauth($r,$cite,$entry);
           }
+          $self->append($r);
+          next;  # move on
         }
       }
+      # use title etc to search.crossref.org
+      my $date; my $atitle; my $jtitle;
+      if (exists $cite->{'publication-date'}->{'year'}) {$date = $cite->{'publication-date'}->{'year'};}
+      if (exists $cite->{'work-title'}->{'title'}) {$atitle = $cite->{'work-title'}->{'title'};}
+      if (exists $cite->{'journal-title'}) {$jtitle = $cite->{'journal-title'};}
+      my $auth = _orcid_getauth($cite);
+      my $temp=$auth.' '.$date.' '.$atitle.' '.$jtitle;
+      if (length($temp)>10 && length($date)>0 && length($atitle)+length($auth)>0) { # we have a potentially useful search string
+        my $r = Bib::CrossRef->new;
+        $r->parse_text($temp);
+        if ($r->score >= 1) {
+          # found an ok match, lets use it
+          $self->append($r);
+          next; # move on
+        }
+      }
+      # for a poor match, try to extract rest of info from orcid
+      my $r = Bib::CrossRef->new;
+      $r->_setdate($date); $r->_setatitle($atitle); $r->_setjtitle($jtitle);
+      if (exists $cite->{'work-type'}) {$r->_setgenre($cite->{'work-type'});}
+      $self->_orcid_setauth($r,$cite);
+      $self->_bibtex_parse($r,$entry);
       $r->_setscore(1);
-      $r->_setquery($auth." ".$temp);
+      $r->_setquery($temp);
       # add manually constructed record
       $self->append($r);
     }
@@ -455,6 +458,7 @@ sub add_orcid {
   }
 }
 
+####################################################################################
 sub _find_pubmed {
   my $c = shift @_;
   my $name = shift @_;
@@ -467,6 +471,7 @@ sub _find_pubmed {
   return undef;
 }
 
+####################################################################################
 sub add_pubmed {
   # add results from a pubmed query
   my ($self,$q) = @_;
@@ -535,6 +540,101 @@ sub add_pubmed {
   $self->_err("Problem with http://eutils.ncbi.nlm.nih.gov: ".$res->status_line);
 }
 
+####################################################################################
+sub _rem_brackets {
+  # remove {} brackets from bibtex entry
+  my $self = shift @_; my $str = shift @_;
+  
+  $str =~ s/[\{\}]//g;
+  $str =~ s/\\textquotesingle/\\'/g;
+  return $str
+}
+
+####################################################################################
+sub _bibtex_setauth {
+  # use author list array to populate our author details
+  my $self = shift @_; my $r = shift @_; my $entry = shift @_;
+
+  if ((!defined $r) || (!defined $entry) || (!$entry->parse_ok)) {return;}
+
+  my @authors = $entry->author;
+  my $count = 0;
+  foreach my $author (@authors) {
+    $count++;
+    $r->_setauth($count,$self->_rem_brackets($author->first).' '.$self->_rem_brackets($author->last));
+  };
+  $r->_setauthcount($count);
+}
+
+####################################################################################
+sub _bibtex_parse {
+  # use data in a bibtext entry to populate citation
+  my $self = shift @_; my $r = shift @_; my $entry = shift @_;
+  
+  if ((!defined $r) || (!defined $entry) || (!$entry->parse_ok)) {return;}
+  
+  my $genre = lc($entry->type);
+  if (($genre eq "inproceedings") || ($genre eq "proceedings")) {
+    $r->_setgenre("proceeding");
+  } else {
+    $r->_setgenre($genre);
+  }
+  if (defined $entry->field('title')) { $r->_setatitle($self->_rem_brackets($entry->field('title')));}
+  if (defined $entry->author) {$self->_bibtex_setauth($r,$entry);}
+  if (defined $entry->field('url')) {$r->_seturl($entry->field('url'))};
+  if (defined $entry->field('year')) {$r->_setdate($entry->field('year'))};
+  if (defined $entry->field('volume')) {$r->_setvolume($entry->field('volume'))};
+  if (defined $entry->field('issue')) {$r->_setissue($entry->field('issue'))};
+  if (defined $entry->field('pages')) {
+    my $pages = $entry->field('pages');
+    (my $s, my $e) = ($pages =~ /([0-9]+)-+([0-9]+)/ );
+    if (defined $s) { $r->_setspage($s); }
+    if (defined $e) { $r->_setepage($e); }
+  }
+  if (defined $entry->field('journal')) {
+    $r->_setjtitle($self->_rem_brackets($entry->field('journal')));
+  } elsif (defined $entry->field('booktitle')) {
+    $r->_setjtitle($self->_rem_brackets($entry->field('booktitle')));
+  }
+  if (defined $entry->field('doi')) {$r->_setdoi($entry->field('doi'));}
+}
+
+####################################################################################
+sub add_bibtex {
+  # read references from a bibtex file, takes file handle as input
+  my $self = shift @_; my $fh = shift @_;
+  my $opt = shift @_;  # options, =1 then use crossref to try to resolve DOIs
+  if (!defined $opt) {$opt=1;}  # $opt defaults to 1
+  
+  my $parser = BibTeX::Parser->new($fh);
+  while (my $entry = $parser->next ) {
+    if (!$entry->parse_ok) { next; } # problem, move on
+    my $doi = $entry->field('doi');
+    if ((defined $doi) && (length($doi)>5) && ($opt)) {
+      # we have a DOI and $opt != 0, use DOI to get rest of citation
+      my $r = Bib::CrossRef->new;
+      $r->parse_text($doi);
+      if ($r->score >=1) {
+        if (!defined $r->authcount || $r->authcount==0) {
+          # shouldn't happen, but sometimes doi data lacks authors.  since we have bibtex data, use it
+          $self->_bibtex_setauth($r,$entry);
+        }
+        $self->append($r);
+        next;  # move on
+      }
+    }
+    my $r = Bib::CrossRef->new;
+    # use bibtex to create citation
+    $self->_bibtex_parse($r,$entry);
+    $r->_setscore(1);
+    $r->_setquery($r->date." ".$r->atitle." ".$r->jtitle);
+    # add manually constructed record
+    $self->append($r);
+  }
+  $self->_split_duplicates();
+}
+
+####################################################################################
 sub add_fromfile {
   # read free text references from a file, one reference per line
   # takes file handle as input
@@ -549,6 +649,7 @@ sub add_fromfile {
   $self->add_details(@cites);
 }
 
+####################################################################################
 sub num {
   # number of references with DOIs
   my $self = shift @_;
@@ -565,12 +666,14 @@ sub num_nodoi {
   return $len;
 }
 
+####################################################################################
 sub getref {
  # get i'th reference with a DOI
  my ($self, $i) = @_;
  return ${$self->{refs}}[$i];
 }
 
+####################################################################################
 sub getref_nodoi {
  # get i'th reference without a DOI
  my ($self, $i) = @_;
@@ -578,6 +681,7 @@ sub getref_nodoi {
  return ${$self->{nodoi_refs}}[$i];
 }
 
+####################################################################################
 sub print {
   # display a list of references
   my $self = shift @_;
@@ -595,6 +699,7 @@ sub print {
   return $out;
 }
 
+####################################################################################
 sub print_nodoi {
   # display a list of references
   my $self = shift @_;
@@ -608,14 +713,16 @@ sub print_nodoi {
     $out .= $self->getref_nodoi($i)->print($i+1);
   }
   if ($self->{html}) {$out.=$self->getref_nodoi(0)->printfooter;}
+  return $out;
 }
 
+####################################################################################
 sub send_resp {
   # generate simple web page with results ...
   
   my $self = shift @_;
 
-  if ($self->num==0 && $self->num_nodoi==0) {return ''};
+  if ($self->num==0 && $self->num_nodoi==0) {return 'No Results'};
   my $html = $self->{html};
   $self->sethtml; # force use of html
   my $out='';
@@ -753,6 +860,13 @@ Uses PubMed API to extract citations listed in response to a query. E.g.
  
 Given a array of strings, one per citation, tries to resolve these into full citations.
 
+=head2 add_bibtex
+
+ $refs->add_bibtex($fh, $opt);
+ 
+Given a file handle to a file containing bibtex entries, imports these citations.  If a citation has a DOI and $opt is non-zero (the default), this
+will be used to try to obtain the full citation from crossref.org.
+
 =head2 add_fromfile
 
  $refs->add_fromfile($fh);
@@ -823,7 +937,7 @@ Adds a Bib::CrossRef to end of a Bib::Tools list of objects
 You can export the following functions if you do not want to use the object orientated interface:
 
 sethtml clearhtml add_details add_google add_google_search add_orcid add_fromfile add_dblp add_pubmed
-send_resp print print_nodoi num num_nodoi getref getref_nodoi append
+send_resp print print_nodoi num num_nodoi getref getref_nodoi append add_bibtex
 
 The tag C<all> is available to easily export everything:
  
@@ -837,33 +951,40 @@ A simple web interface to Bib::Tools is contained in the examples folder.  This
 
  <!DOCTYPE HTML>
  <html><head><meta http-equiv="Content-Type" content="text/html;charset=UTF-8"></head><body>
+ <div style="position:relative; height:600px; padding:10px; border:1px solid">
  <h3>Import References</h3>
- <form action="handle_query.pl" method="POST" id="in">
+ <form action="handle_query.pl" method="POST" id="in" enctype="multipart/form-data">
  <table>
  <tr><td>Use ORCID id:<br><small style="color:#C0C0C0">e.g. 0000-0003-4056-4014</small></td>
- <td><INPUT type="text" name="orcid" size="128"></p></td>
- </tr>
- <tr><td colspan=2>(to import from Scopus, follow these <a href="http://orcid.scopusfeedback.com/">instructions</a>)</td></tr>
+ <td><INPUT type="text" name="orcid" size="128"></p></td></tr>
+ <tr><td colspan=2>(to import from Scopus, follow these <a href="http://orcid.scopusfeedback.com/">instructions</a>, and for Web of Science/ResearcherId follow <a href="http://wokinfo.com/researcherid/integration/">these</a>)</td></tr>
+
+ <tr><td width="150px">Use DBLP XML page:<br>
+ <small style="color:#C0C0C0">e.g. http://www.informatik.uni-trier.de/~ley/pers/xx/l/Leith:Douglas_J=</small></td>
+ <td> <INPUT type="text" name="dblp" size="128"></td></tr>
+
+ <tr><td>Use PubMed query:<br><small style="color:#C0C0C0">e.g. mills kh[author]</small></td>
+ <td> <INPUT type="text" name="query" size="128"></td></tr>
+
+ <tr><td>BibTeX file:</td>
+ <td> <INPUT type="file" name="bibtex"></td></tr>
+
  <tr><td>Use Google Scholar personal page:<br>
  <small style="color:#C0C0C0">e.g. http://scholar.google.com/citations?user=n8dX1fUAAAAJ</small></td>
- <td><INPUT type="text" name="google" size="128"></p></td>
- </tr>
+ <td><INPUT type="text" name="google" size="128"></p></td></tr>
+
  <tr><td>Use Google Scholar search page:<br>
  <small style="color:#C0C0C0">e.g. http://scholar.google.com/scholar?q=andr%C3%A9s+garcia+saavedra</small></td>
- <td><INPUT type="text" name="google2" size="128"></p></td>
- </tr>
- <tr><td>Use DBLP page:<br>
- <small style="color:#C0C0C0">e.g. 'http://www.informatik.uni-trier.de/~ley/pers/xx/l/Leith:Douglas_J=</small></td>
- <td> <INPUT type="text" name="dblp" size="128"></td></tr>
- <tr><td>Use PubMed query:<br>
- <small style="color:#C0C0C0">e.g. mills kh[author]</small></td>
- <td> <INPUT type="text" name="query" size="128"></td></tr>
- </table>
- <p>Enter references, one per line:</p><textarea name="refs" rows="10" cols="128" form="in"></textarea><br>
- <INPUT type="submit" value="Submit">(can be slow, be patient)
- </form>
- </body></html>
+ <td><INPUT type="text" name="google2" size="128"></p></td></tr>
 
+ </table>
+ <p>Enter references, one per line (free form text):</p>
+ <textarea name="refs" rows="10" cols="128" form="in"></textarea><br>
+ <INPUT type="submit" value="Submit">(can be slow, be patient)
+ </form></div>
+ <div style="position: absolute; bottom: 5px;"><small>Source: <a href="http://search.cpan.org/~dougleith/Bib-Tools/">Bib::Tools</a></div>
+ </body></html>
+ 
 =head2 handle_query.pl
 
  #!/usr/bin/perl
@@ -910,6 +1031,11 @@ A simple web interface to Bib::Tools is contained in the examples folder.  This
  if (length($pubmed) > 5) {
    $refs->add_pubmed($pubmed);
  }
+
+ $filename = scalar $q->param('bibtex');
+ $tmpfilename = $q->tmpFileName($filename);
+ open my $fh, "<", $tmpfilename;
+ $refs->add_bibtex($fh);
  
  my @values = $q->multi_param('refs');
  foreach my $value (@values) {
@@ -948,7 +1074,7 @@ A simple web interface to Bib::Tools is contained in the examples folder.  This
 
 =head1 VERSION
  
-Ver 0.14
+Ver 0.15
  
 =head1 AUTHOR
  
